@@ -14,7 +14,7 @@ import base64
 import math
 
 
-def _rc(*arg):
+def _rc(arg):
     cov_mat = np.cov(arg)
     var_mat = np.diagonal(cov_mat)**0.5
     var_mat[var_mat == 0] = 1
@@ -30,7 +30,7 @@ def _preview_process(data, value_round):
     '''
     预览处理
     '''
-    data = np.array(data, dtype=np.float)
+    data = np.array(data, dtype=float)
     
     name_list = ['平均值', '中位数', '方差', '标准差', '最大值', '最小值', '偏度', '峰度']
     value_list = []
@@ -64,18 +64,19 @@ def _preview_process(data, value_round):
     table = '<h1 style="padding-left:50px;">数据信息</h1>' + table % tuple(value_list) + '</table>'
     
     
-    data = data.T
+    data = np.ascontiguousarray(data.T)
     num = data.shape[0]
     plt.figure(figsize=(9, 3 * num))
     for i in range(num):
         q1, q2, q3 = np.percentile(data[i], [25, 50, 75])
         plt.scatter(mean_[i], i+1, marker='o', color='white', s=30, zorder=3)
         plt.hlines(i+1, q1, q3, color='k', linestyle='-', lw=1)
-    ax = plt.violinplot(data.tolist(), showextrema=False, vert=False)
+    bx = plt.violinplot(data.tolist(), showextrema=False, vert=False)
     plt.title('分布图')
     
     buffer = BytesIO()
     plt.savefig(buffer)
+    plt.close()
     plot_data = buffer.getvalue()
     imb = base64.b64encode(plot_data)
     ims = imb.decode()
@@ -83,7 +84,7 @@ def _preview_process(data, value_round):
     im1 = '<div align="center"><img src="%s"></div>' % imd
     im1 = '<br></br><h2 style="padding-left:50px; border-top:1px solid #ccc">密度分布</h2>' + im1
     
-    cov_mat = _rc(*data)
+    cov_mat = _rc(data)
     matrix = '<table border="0"><caption></caption>'
     
     for i in range(num):
@@ -98,6 +99,7 @@ def _preview_process(data, value_round):
     
     buffer = BytesIO()
     plt.savefig(buffer)
+    plt.close()
     plot_data = buffer.getvalue()
     imb = base64.b64encode(plot_data)
     ims = imb.decode()
@@ -114,6 +116,7 @@ def _preview_process(data, value_round):
     
     buffer = BytesIO()
     plt.savefig(buffer)
+    plt.close()
     plot_data = buffer.getvalue()
     imb = base64.b64encode(plot_data)
     ims = imb.decode()
@@ -172,7 +175,7 @@ def preview(data, value_round=3):
     display(HTML(root))
 
 
-def length_pad(seq, maxlen=None, value=0, padding='pre', dtype='int32'):
+def length_pad(seq, maxlen=None, value=0, padding='pre', dtype=float):
     '''
     填充二维列表，使得每行长度都为maxlen
     
@@ -182,7 +185,7 @@ def length_pad(seq, maxlen=None, value=0, padding='pre', dtype='int32'):
     maxlen：整型，可选，每行的最大长度，默认为原二维列表最大的长度
     value：数类型，可选，填充值，默认为0
     padding：字符串类型，可选，填充位置，'pre'代表从前面填充，'post'代表从后面填充，默认为'pre'
-    dtype：可选，输出的元素类型，默认为'int32'
+    dtype：可选，输出的元素类型，默认为float
     
     返回
     ----
@@ -197,7 +200,7 @@ def length_pad(seq, maxlen=None, value=0, padding='pre', dtype='int32'):
     maxlen: int, callable, the maximum length of each row, default = the maximum length of the original 2-D list
     value: num, callable, padding value, default=0
     padding: str, callable, padding location, 'pre' means padding from the front and 'post' from the back, default='pre'
-    dtype: callable, the element type of the output, default='int32'
+    dtype: callable, the element type of the output, default=float
     
     Return
     ------
@@ -286,7 +289,7 @@ def sample_pad(data, index=0, padding=None):
     if not padding:
         padding = [np.nan] * (len(data[0][0]) - 1)
     else:
-        padding = list(padding)
+        padding = list([padding])
     
     for i in range(len(data)):
         data_part = np.array(data[i], dtype=np.object)
@@ -297,7 +300,7 @@ def sample_pad(data, index=0, padding=None):
         different_set_list = np.array([list(time_set - set(result[i][:, index]))], dtype=np.object).T
         num = len(different_set_list)
         padding_new = np.array(padding * num, dtype=np.object).reshape(num, -1)
-        different_set_list = np.hstack((different_set_list, padding_new))
+        different_set_list = np.hstack((padding_new[:, :index], different_set_list, padding_new[:, index:]))
         result[i] = np.vstack((result[i], different_set_list))
     
     return result
@@ -305,14 +308,14 @@ def sample_pad(data, index=0, padding=None):
 
 def shuffle(*arg):
     '''
-    以相同方法打乱多个序列或打乱一个序列
+    打乱一个序列或以相同方法打乱多个序列
     
     返回
     ----
     一个ndarray
     
     
-    shuffle multiple sequences in the same way or shuffle a sequences
+    Shuffle a sequence or shuffle multiple sequences in the same way
     
     Return
     ------
@@ -360,6 +363,8 @@ def data_split(data, targets, train_size=None, test_size=None, shuffle=True, ran
     ------
     tuple, (train_data, train_target, validation_data, validation_target)
     '''
+    data = np.array(data)
+    targets = np.array(targets)
     if not (train_size or test_size):
         train_size = 0.75
     elif test_size:
@@ -370,7 +375,6 @@ def data_split(data, targets, train_size=None, test_size=None, shuffle=True, ran
     
     if shuffle:
         np.random.seed(random_state)
-        
         state = np.random.get_state()
         np.random.shuffle(data)
         np.random.set_state(state)
@@ -385,53 +389,6 @@ def data_split(data, targets, train_size=None, test_size=None, shuffle=True, ran
     return train_data, train_target, validation_data, validation_target
 
 
-def dataloader(data, targets, choose_rate=0.3, is_shuffle=True, random_state=None):
-    '''
-    数据随机生成器
-    
-    参数
-    ----
-    data：数据
-    targets：指标
-    choose_rate：浮点数类型，可选，生成率，范围为[0, 1]，默认为0.3
-    is_shuffle：布尔类型，可选，True表示打乱数据，False表示不打乱数据，默认为True
-    random_state：整型，可选，随机种子
-    
-    返回
-    ----
-    生成器
-    
-    
-    Data Random Generator
-    
-    Parameters
-    ----------
-    data: data
-    targets: targets
-    choose_rate: float, callable, generation rate whose range is [0, 1], default=0.3
-    is_shuffle: bool, callable, 'True' will shuffle the data, 'False' will not, default = True
-    random_state: int, callable, random seed
-    
-    Return
-    ------
-    generator
-    '''
-    data = np.array(data)
-    targets = np.array(targets)
-    
-    if is_shuffle:
-        np.random.seed(random_state)
-        data, targets = shuffle(data, targets)
-    num = len(data)
-    choose_rate = int(num * choose_rate)
-    times = int(math.ceil(num / choose_rate))
-    
-    for i in range(times):
-        loc_1 = i * choose_rate
-        loc_2 = (i + 1) * choose_rate
-        yield data[loc_1: loc_2], targets[loc_1: loc_2]
-
-
 def kfold(data, targets, n, k=5):
     '''
     参数
@@ -457,6 +414,8 @@ def kfold(data, targets, n, k=5):
     ------
     tuple, (train_data, train_target, validation_data, validation_target)
     '''
+    data = np.array(data)
+    targets = np.array(targets)
     num_validation_samples = len(data) // k
     
     validation_data = data[num_validation_samples * n:
@@ -472,7 +431,57 @@ def kfold(data, targets, n, k=5):
     return train_data, train_targets, validation_data, validation_targets
 
 
-def standard(data, index=None, mean=True, var=True):
+def dataloader(data, targets, choose_rate=0.3, shuffle=True, random_state=None):
+    '''
+    数据随机生成器
+    
+    参数
+    ----
+    data：数据
+    targets：指标
+    choose_rate：浮点数类型，可选，生成率，即一次生成数据量在原数据量的占比，范围为[0, 1]，默认为0.3
+    shuffle：布尔类型，可选，True表示打乱数据，False表示不打乱数据，默认为True
+    random_state：整型，可选，随机种子
+    
+    返回
+    ----
+    生成器
+    
+    
+    Data Random Generator
+    
+    Parameters
+    ----------
+    data: data
+    targets: targets
+    choose_rate: float, callable, generation rate (the proportion of data generated at one time in the original data) whose range is [0, 1], default=0.3
+    shuffle: bool, callable, 'True' will shuffle the data, 'False' will not, default = True
+    random_state: int, callable, random seed
+    
+    Return
+    ------
+    generator
+    '''
+    data = np.array(data)
+    targets = np.array(targets)
+    
+    if shuffle:
+        np.random.seed(random_state)
+        state = np.random.get_state()
+        np.random.shuffle(data)
+        np.random.set_state(state)
+        np.random.shuffle(targets)
+    num = len(data)
+    choose_rate = int(num * choose_rate)
+    times = int(math.ceil(num / choose_rate))
+    
+    for i in range(times):
+        loc_1 = i * choose_rate
+        loc_2 = (i + 1) * choose_rate
+        yield data[loc_1: loc_2], targets[loc_1: loc_2]
+
+
+def standard(data, mean=True, var=True, index=None):
     '''
     标准化数据
     z = (x - u) / s
@@ -482,13 +491,13 @@ def standard(data, index=None, mean=True, var=True):
     参数
     ----
     data：2-D的ndarray数据
-    index：列表形式，可选，需要进行标准化的列的索引，默认为全部
-    mean：布尔类型，可选，是否将均值调整为0
-    var：布尔类型，可选，是否将方差调整为1
+    mean：布尔类型或ndarray，可选，布尔类型决定是否将均值调整为0，ndarray用于设定不同的均值
+    var：布尔类型或ndarray，可选，是否将方差调整为1，ndarray用于设定不同的方差
+    index：列表类型，可选，需要进行标准化的列的索引，默认为全部
     
     返回
     ----
-    2-D ndarray
+    tuple
     
     
     Standardize data
@@ -499,9 +508,9 @@ def standard(data, index=None, mean=True, var=True):
     Parameters
     ----------
     data: 2-D ndarray
+    mean: bool or ndarray, callable, bool decides if adjust the mean value to 0, ndarray is used to set different means
+    var: bool or ndarray, callable, bool decides if adjust the variance to 0, ndarray is used to set different variances
     index: list, callable, index of columns need to be standardized, defalut to all
-    mean: bool, callable, if adjust the mean value to 0
-    var: bool, callable, if adjust the variance to 0
     
     Return
     ------
@@ -510,40 +519,55 @@ def standard(data, index=None, mean=True, var=True):
     data=np.array(data, dtype=np.float)
     
     if index:
-        if mean:
+        if mean is True:
             mean = data[:, index].mean(axis=0)
-        else:
-            mean = 0.0
+        elif mean is None or mean is False:
+            mean = np.zeros(len(index))
         data[:, index] -= mean
         
-        if var:
-            std = data[:, index].std(axis=0)
-            std_zero_indices = np.nonzero(std == 0)
+        if not var is None and not var is False:
+            if var is True:
+                var = data[:, index].std(axis=0)
+            else:
+                var = np.array(var)
+            std_zero_indices = np.nonzero(var == 0)
+            std = var.copy()
             std[std==0] = 1.0
             data[:, index] /= std
             if list(std_zero_indices[0]):
                 for i in std_zero_indices[0]:
                     data[:, index][:, i] *= 0
-    else:
-        if mean:
-            mean = data.mean(axis=0)
         else:
-            mean = 0.0
+            std = 0
+    
+    else:
+        if mean is True:
+            mean = data.mean(axis=0)
+        elif mean is None or mean is False:
+            mean = np.zeros(data.shape[1])
+        else:
+            mean = np.array(mean)
         data -= mean
         
-        if var:
-            std = data.std(axis=0)
-            std_zero_indices = np.nonzero(std == 0)
+        if not var is None and not var is False:
+            if var is True:
+                var = data.std(axis=0)
+            else:
+                var = np.array(var)
+            std_zero_indices = np.nonzero(var == 0)
+            std = var.copy()
             std[std==0] = 1.0
             data /= std
             if list(std_zero_indices[0]):
                 for i in std_zero_indices[0]:
                     data[:, i] *= 0
+        else:
+            std = 0
     
-    return data
+    return data, mean, std
 
 
-def minmax(data, index=None, feature_range=(0, 1)):
+def minmax(data, feature_range=(0, 1), min_max=None, index=None):
     '''
     归一化数据
     X_std = (X - X.min(axis=0)) / (X.max(axis=0) - X.min(axis=0))
@@ -552,43 +576,47 @@ def minmax(data, index=None, feature_range=(0, 1)):
     参数
     ----
     data：2-D的ndarray数据
-    index：列表形式，可选，需要进行标准化的列的索引，默认为全部
-    feature_range：元组形式，可选，需要转换的范围，默认为(0, 1)
+    feature_range：元组类型，可选，需要转换的范围，默认为(0, 1)
+    min_max：元组类型，可选，用于设定最大最小值
+    index：列表类型，可选，需要进行标准化的列的索引，默认为全部
     
     返回
     ----
-    2-D ndarray
+    元组，(归一化后的数据, (最小值，最大值))
     
     
-    Normalized data
+    Normalize data
     X_std = (X - X.min(axis=0)) / (X.max(axis=0) - X.min(axis=0))
     X_new = X_std * (feature_range[1] - feature_range[0]) + feature_range[0]
     
     Parameters
     ----------
     data: 2-D的ndarray数据
-    index: list, callable, index of columns need to be standardized, defalut to all
     feature_range: tuple, callabel, final range of transformed data
+    min_max: tuple, callable, set min and max values
+    index: list, callable, index of columns need to be standardized, defalut to all
     
     Return
     ------
-    2-D ndarray
+    tuple, (Normalized data, (miniment, maximent))
     '''
     data=np.array(data, dtype=np.float)
     
     if index:
-        min_data = data[:, index].min(axis=0)
-        length = data[:, index].max(axis=0) - min_data
-        data[:, index] = (data[:, index] - min_data) / length
+        if not min_max:
+            min_max = (data[:, index].min(axis=0), data[:, index].max(axis=0))
+        length = min_max[1] - min_max[0]
+        data[:, index] = (data[:, index] - min_max[0]) / length
         data[:, index] = data[:, index] * (feature_range[1] - feature_range[0]) + feature_range[0]
     
     else:
-        min_data = data.min(axis=0)
-        length = data.max(axis=0) - min_data
-        data = (data - min_data) / length
+        if not min_max:
+            min_max = (data.min(axis=0), data.max(axis=0))
+        length = min_max[1] - min_max[0]
+        data = (data - min_max[0]) / length
         data = data * (feature_range[1] - feature_range[0]) + feature_range[0]
     
-    return data
+    return data, min_max
 
 
 def normalizer(data, index=None):
@@ -616,98 +644,85 @@ def normalizer(data, index=None):
     ------
     2-D ndarray
     '''
-    data = np.array(data, dtype=np.float)
+    data = np.array(data, dtype=float)
     
     if index:
         distance_list = e_distances(data[:, index])
         distance_list[distance_list == 0] = 1
-        print(distance_list)
-        data[:, index] /= distance_list
+        data[:, index] /= np.array([distance_list]).T
     
     else:
         distance_list = e_distances(data)
         distance_list[distance_list == 0] = 1
-        print(distance_list)
-        data /= distance_list
+        data /= np.array([distance_list]).T
     
     return data
 
 
-class Minmax():
+def pca(data, n=None, eig_vector=None):
     '''
-    归一化数据
-    以一组数据为标准，将其他数据按照该组数据的最大值和最小值进行归一化
-    X_std = (X - X.min(axis=0)) / (X.max(axis=0) - X.min(axis=0))
-    X_new = X_std * (feature_range[1] - feature_range[0]) + feature_range[0]
+    主成分分析
     
     参数
     ----
-    data：2-D的ndarray数据
+    data：二维数组，数据
+    n：整型或浮点数类型，可选，当n >= 1时，表示降至n维，当0< n < 1时，表示降维至累计方差贡献率 >= n，默认不降维
+    eig_vector：元组类型，可选，用于设定不同的特征向量，当设置该参数时，累计方差贡献率无意义，默认为None
     
-    属性
+    返回
     ----
-    minimum：1-D ndarray，最小值
-    maximum：1-D ndarray，最大值
-    length：1-D ndarray，最大值 - 最小值
+    元组，(转换后的数据, (累计方差贡献率, 各主成分方差, 各主成分方差贡献率), 特征向量)
     
     
-    Normalized data
-    Take a group of data as the standard, other data will be normalized according to the minimum and maximum value of this group of data
-    X_std = (X - X.min(axis=0)) / (X.max(axis=0) - X.min(axis=0))
-    X_new = X_std * (feature_range[1] - feature_range[0]) + feature_range[0]
+    Principal Component Analysis
     
-    Parameter
-    ---------
-    data: 2-D的ndarray数据
-    
-    Attributes
+    Parameters
     ----------
-    minimum: 1-D ndarray
-    maximum: 1-D ndarray
-    length: 1-D ndarray, maximum - minimum
+    data：2-D array, data
+    n: int or float, callable, when n > = 1, it means to reduce the dimension to n; when 0 < n < 1, it means to reduce the dimension to cumulative variance ratio > = n, and it is not reduced by default
+    eig_value_vector: tuple, callable, set different eigenvectors, when this parameter is set, the cumulative variance ratio is meaningless, default=None
+    
+    Return
+    ------
+    tuple, (transformed data, (cumulative variance ratio, variance, variance ratio), eigenvectors)
     '''
-    def __init__(self, data):
-        data=np.array(data, dtype=np.float)
-        
-        self.minimum = data.min(axis=0)
-        self.maximum = data.max(axis=0)
-        self.length = self.maximum - self.minimum
+    data = np.array(data, dtype=float)
     
-    
-    def transform(self, data, index=None, feature_range=(0, 1)):
-        '''
-        参数
-        ----
-        data：2-D的ndarray数据
-        index：列表形式，可选，需要进行标准化的列的索引，默认为全部
-        feature_range：元组形式，可选，需要转换的范围，默认为(0, 1)
+    if eig_vector is None or eig_vector is False:
+        cov_mat = np.cov(data.T)
+        eig_value, eig_vector = np.linalg.eig(cov_mat)
         
-        返回
-        ----
-        2-D ndarray
-        
-        
-        Parameters
-        ----------
-        data: 2-D的ndarray数据
-        index: list, callable, index of columns need to be standardized, defalut to all
-        feature_range: tuple, callabel, final range of transformed data
-        
-        
-        Return
-        ------
-        2-D ndarray
-        '''
-        data=np.array(data, dtype=np.float)
-        if index:
-            data[:, index] = (data[:, index] - self.minimum[index]) / self.length[index]
-            data[:, index] = data[:, index] * (feature_range[1] - feature_range[0]) + feature_range[0]
+        sort_index = np.flipud(np.argsort(eig_value))
+        eig_value = eig_value[sort_index]
+        eig_vector = eig_vector[:, sort_index]
         
+        eig_ratio = eig_value / eig_value.sum()
+        contri = 0
+        if not n is None and not n is False:
+            if n >= 1:
+                n = int(n)
+                contri = eig_ratio[:n].sum()
+            
+            elif n < 1:
+                for i in range(eig_value.shape[0]):
+                    contri += eig_ratio[i]
+                    if contri >= n:
+                        n = i + 1
+                        break
+            eig_value = eig_value[:n]
+            eig_ratio = eig_ratio[:n]
+            eig_vector = eig_vector[:, :n]
         else:
-            data = (data - self.minimum) / self.length
-            data = data * (feature_range[1] - feature_range[0]) + feature_range[0]
-        
-        return data
+            contri = 1
+    
+    else:
+        contri = None
+        eig_value = None
+        eig_ratio = None
+    
+    data = np.dot(data, eig_vector)
+    
+    return data, (contri, eig_value, eig_ratio), eig_vector
 
 
 class RC():
@@ -738,7 +753,7 @@ class RC():
         if len(arg.shape) != 2:
             raise Exception("Input list should be 1-D.")
         else:
-            self.rc_mat = _rc(*arg)
+            self.rc_mat = _rc(arg)
     
     
     def __img_process(self, index, cmap):
@@ -791,97 +806,4 @@ class RC():
         cmap: str, callable, color board, default='Blues'
         '''
         self.__img_process(index=index, cmap=cmap)
-        plt.savefig(filename)
-
-
-class Standard():
-    '''
-    标准化数据
-    以一组数据为标准，将其他数据按照该组数据的方差和均值进行标准化
-    z = (x - u) / s
-    z：新数据；  x：原数据；  u：均值；  s：方差
-    如果某一列数据完全相同（即方差s=0），则该列数据全部归零
-    
-    参数
-    ----
-    data：2-D的ndarray数据
-    mean：布尔类型，可选，是否将均值调整为0
-    var：布尔类型，可选，是否将方差调整为1
-    
-    属性
-    ----
-    mean：1-D的ndarray，数据的均值
-    var：1-D的ndarray，数据的方差
-    
-    
-    Standardize data
-    Take a group of data as the standard, other data will be standardized according to the variance and mean value of this group of data
-    z = (x - u) / s
-    z: new data;  x: origin data;  u: mean value;  s: variance
-    if data in one column are the same(s=0), data in this column will be turned to 0
-    
-    Parameters
-    ----------
-    data: 2-D ndarray
-    mean: bool, callable, if adjust the mean value to 0
-    var: bool, callable, if adjust the variance to 0
-    
-    Attributes
-    ----------
-    mean: 1-D ndarray, mean values of the data
-    var: 1-D ndarray, variances of the data
-    '''
-    def __init__(self, data, mean=True, var=True):
-        data=np.array(data, dtype=np.float)
-        
-        if mean:
-            self.mean = data.mean(axis=0)
-        else:
-            self.mean = False
-        
-        if var:
-            self.var = data.var(axis=0)
-            std_zero_indices = np.nonzero(self.var == 0)
-            self.var[self.var==0] = 1.0
-            self.__std_zero_indices = std_zero_indices[0]
-        else:
-            self.var = False
-    
-    
-    def transform(self, data, index=None):
-        '''
-        参数
-        ----
-        data：2-D的ndarray数据
-        index：列表形式，可选，需要进行标准化的列的索引，默认为全部
-        
-        返回
-        ----
-        2-D ndarray
-        
-        
-        Parameters
-        ----------
-        data: 2-D ndarray
-        index: list, callable, index of columns need to be standardized, defalut to all
-        
-        Return
-        ------
-        2-D ndarray
-        '''
-        data=np.array(data, dtype=np.float)
-        if index:
-            if type(self.mean).__name__ == 'ndarray':
-                data[:, index] -= self.mean[index]
-            
-            if type(self.var).__name__ == 'ndarray':
-                data[:, index] /= self.var[index]
-        
-        else:
-            if type(self.mean).__name__ == 'ndarray':
-                data -= self.mean
-            
-            if type(self.var).__name__ == 'ndarray':
-                data /= self.var
-        
-        return data
\ No newline at end of file
+        plt.savefig(filename)
\ No newline at end of file
